@@ -30,7 +30,6 @@ from typing import Union
 from typing import Optional
 from typing import Callable
 import os
-import os.path as path
 import shutil
 import sqlite3
 import uuid
@@ -99,8 +98,10 @@ class ImeProperties:
         sqlstr = 'SELECT attr, val FROM main.ime;'
         try:
             results = db.execute(sqlstr).fetchall()
-        except:
-            LOGGER.exception('Cannot get ime properties from database')
+        except Exception as error: # pylint: disable=broad-except
+            LOGGER.exception(
+                'Cannot get ime properties from database: %s: %s',
+                error.__class__.__name__, error)
         for result in results:
             self.ime_property_cache[result[0]] = result[1]
 
@@ -115,7 +116,7 @@ class ImeProperties:
         return ''
 
     def __str__(self) -> str:
-        return 'ime_property_cache = %s' %repr(self.ime_property_cache)
+        return f'ime_property_cache = {repr(self.ime_property_cache)}'
 
 class TabSqliteDb:
     '''Phrase database for tables
@@ -150,7 +151,7 @@ class TabSqliteDb:
             user_db: str = '',
             create_database: bool = False,
             unit_test: bool = False) -> None:
-        global DEBUG_LEVEL
+        global DEBUG_LEVEL # pylint: disable=global-statement
         try:
             DEBUG_LEVEL = int(str(os.getenv('IBUS_TABLE_DEBUG_LEVEL')))
         except (TypeError, ValueError):
@@ -163,7 +164,7 @@ class TabSqliteDb:
         if create_database or os.path.isfile(self.filename):
             self.db: sqlite3.dbapi2.Connection = sqlite3.connect(self.filename)
         else:
-            print('Cannot open database file %s' %self.filename)
+            print(f'Cannot open database file {self.filename}')
         try:
             self.db.execute('PRAGMA encoding = "UTF-8";')
             self.db.execute('PRAGMA case_sensitive_like = true;')
@@ -173,8 +174,11 @@ class TabSqliteDb:
             self.db.execute('PRAGMA temp_store = MEMORY;')
             self.db.execute('PRAGMA journal_size_limit = 1000000;')
             self.db.execute('PRAGMA synchronous = NORMAL;')
-        except:
-            LOGGER.exception('Error while initializing database')
+            self.db.execute('PRAGMA busy_timeout = 5000;')
+        except Exception as error: # pylint: disable=broad-except:
+            LOGGER.exception(
+                'Error while initializing database: %s: %s',
+                error.__class__.__name__, error)
         # create IME property table
         self.db.executescript(
             'CREATE TABLE IF NOT EXISTS main.ime (attr TEXT, val TEXT);')
@@ -188,8 +192,8 @@ class TabSqliteDb:
             'name.zh_hk':'',
             'name.zh_tw':'',
             'author':'somebody',
-            'uuid':'%s' % uuid.uuid4(),
-            'serial_number':'%s' % time.strftime('%Y%m%d'),
+            'uuid': f'{uuid.uuid4()}',
+            'serial_number': f'{time.strftime("%Y%m%d")}',
             'icon':'ibus-table.svg',
             'license':'LGPL',
             'languages':'',
@@ -243,8 +247,8 @@ class TabSqliteDb:
         # shared variables in this class:
         self._mlen = int(self.ime_properties.get("max_key_length"))
         self._snum = self.ime_properties.get("serial_number")
-        self._is_chinese = self.is_chinese()
-        self._is_cjk = self.is_cjk()
+        self.is_db_chinese = self.is_chinese()
+        self.is_db_cjk = self.is_cjk()
         self.user_can_define_phrase = (self.ime_properties.get(
             'user_can_define_phrase').lower() == 'true')
 
@@ -252,9 +256,9 @@ class TabSqliteDb:
         self.possible_tabkeys_lengths = self.get_possible_tabkeys_lengths()
         self.startchars = self.get_start_chars()
 
-        tables_path = path.join(ibus_table_location.data_home(), 'tables')
+        tables_path = os.path.join(ibus_table_location.data_home(), 'tables')
         cache_name = os.path.basename(self.filename).replace('.db', '.cache')
-        self.cache_path = path.join(tables_path, cache_name)
+        self.cache_path = os.path.join(tables_path, cache_name)
         if not unit_test:
             self.load_phrases_cache()
 
@@ -276,9 +280,9 @@ class TabSqliteDb:
             #
             # “HOME=/foobar ibus-table-createdb” should not fail if
             # “/foobar” is not writeable.
-            if not path.isdir(tables_path):
+            if not os.path.isdir(tables_path):
                 old_tables_path = os.path.expanduser('~/.ibus/tables')
-                if path.isdir(old_tables_path):
+                if os.path.isdir(old_tables_path):
                     if os.access(os.path.join(
                             old_tables_path, 'debug.log'), os.F_OK):
                         os.unlink(os.path.join(old_tables_path, 'debug.log'))
@@ -291,8 +295,8 @@ class TabSqliteDb:
                     os.symlink(tables_path, old_tables_path)
                 else:
                     os.makedirs(tables_path, exist_ok=True)
-            user_db = path.join(tables_path, user_db)
-            if not path.exists(user_db):
+            user_db = os.path.join(tables_path, user_db)
+            if not os.path.exists(user_db):
                 LOGGER.debug(
                     'The user database %s does not exist yet.', user_db)
             else:
@@ -350,7 +354,7 @@ class TabSqliteDb:
                             os.rename(user_db+'-wal', user_db+'-wal'+timestamp)
                         LOGGER.debug(
                             'Creating a new, empty database "%s".', user_db)
-                        self.init_user_db(user_db)
+                        TabSqliteDb._init_user_db(user_db)
                         LOGGER.debug(
                             'If user phrases were successfully recovered from '
                             'the old, '
@@ -359,16 +363,17 @@ class TabSqliteDb:
                     else:
                         LOGGER.debug(
                             'Compatible database %s found.', user_db)
-                except:
+                except Exception as error: # pylint: disable=broad-except
                     LOGGER.exception(
-                        'Unexpected error trying to find user database')
+                        'Unexpected error trying to find user database: %s: %s',
+                        error.__class__.__name__, error)
 
         # open user phrase database
         try:
             LOGGER.debug(
                 'Connect to the database %s.', user_db)
-            self.db.executescript('''
-                ATTACH DATABASE "%s" AS user_db;
+            self.db.executescript(f'''
+                ATTACH DATABASE "{user_db}" AS user_db;
                 PRAGMA user_db.encoding = "UTF-8";
                 PRAGMA user_db.case_sensitive_like = true;
                 PRAGMA user_db.page_size = 4096;
@@ -377,9 +382,12 @@ class TabSqliteDb:
                 PRAGMA user_db.journal_mode = WAL;
                 PRAGMA user_db.journal_size_limit = 1000000;
                 PRAGMA user_db.synchronous = NORMAL;
-            ''' % user_db)
-        except:
-            LOGGER.debug('Could not open the database %s.', user_db)
+                PRAGMA busy_timeout = 5000;
+            ''')
+        except Exception as error:  # pylint: disable=broad-except
+            LOGGER.exception(
+                'Could not open the database %s: %s: %s',
+                user_db, error.__class__.__name__, error)
             timestamp = time.strftime('-%Y-%m-%d_%H:%M:%S')
             LOGGER.debug('Renaming the incompatible database to "%s".',
                          user_db+timestamp)
@@ -390,9 +398,9 @@ class TabSqliteDb:
             if os.path.exists(user_db+'-wal'):
                 os.rename(user_db+'-wal', user_db+'-wal'+timestamp)
             LOGGER.debug('Creating a new, empty database "%s".', user_db)
-            self.init_user_db(user_db)
-            self.db.executescript('''
-                ATTACH DATABASE "%s" AS user_db;
+            TabSqliteDb._init_user_db(user_db)
+            self.db.executescript(f'''
+                ATTACH DATABASE "{user_db}" AS user_db;
                 PRAGMA user_db.encoding = "UTF-8";
                 PRAGMA user_db.case_sensitive_like = true;
                 PRAGMA user_db.page_size = 4096;
@@ -401,7 +409,8 @@ class TabSqliteDb:
                 PRAGMA user_db.journal_mode = WAL;
                 PRAGMA user_db.journal_size_limit = 1000000;
                 PRAGMA user_db.synchronous = NORMAL;
-            ''' % user_db)
+                PRAGMA busy_timeout = 5000;
+            ''')
         self.create_tables("user_db")
         if self.old_phrases:
             sqlargs_old_phrases: List[Dict[str, Union[str, int]]] = []
@@ -417,8 +426,9 @@ class TabSqliteDb:
             '''
             try:
                 self.db.executemany(sqlstr, sqlargs_old_phrases)
-            except:
-                LOGGER.exception('Error inserting old phrases')
+            except sqlite3.Error as error:
+                LOGGER.exception('Error inserting old phrases: %s: %s',
+                                  error.__class__.__name__, error)
             self.db.commit()
             self.db.execute('PRAGMA wal_checkpoint;')
 
@@ -440,10 +450,10 @@ class TabSqliteDb:
                 tabkeys, phrase, user_freq, database)
         if not tabkeys or not phrase:
             return
-        sqlstr = '''
-        UPDATE %s.phrases SET user_freq = :user_freq
+        sqlstr = f'''
+        UPDATE {database}.phrases SET user_freq = :user_freq
         WHERE tabkeys = :tabkeys AND phrase = :phrase
-        ;''' % database
+        ;'''
         sqlargs = {'user_freq': user_freq,
                    'tabkeys': tabkeys,
                    'phrase': phrase}
@@ -452,8 +462,10 @@ class TabSqliteDb:
             if commit:
                 self.db.commit()
             self.invalidate_phrases_cache(tabkeys)
-        except:
-            LOGGER.exception('Unexpected error updating phrase in user_db.')
+        except sqlite3.Error as error:
+            LOGGER.exception(
+                'Unexpected error updating phrase in user_db: %s: %s',
+                error.__class__.__name__, error)
 
     def sync_usrdb(self) -> None:
         '''
@@ -493,7 +505,8 @@ class TabSqliteDb:
         if DEBUG_LEVEL > 1:
             LOGGER.debug('load_phrases_cache()')
         try:
-            self._phrases_cache = json.load(open(self.cache_path))
+            with open(self.cache_path, encoding='utf-8') as f:
+                self._phrases_cache = json.load(f)
             snum = self._phrases_cache.get('serial_number')
             if not snum or (snum != self._snum):
                 self._phrases_cache = {}
@@ -505,8 +518,10 @@ class TabSqliteDb:
             if DEBUG_LEVEL > 1:
                 LOGGER.debug(
                     'Permission error reading %s', self.cache_path)
-        except:
-            LOGGER.debug('Unknown error reading %s', self.cache_path)
+        except Exception as error: # pylint: disable=broad-except
+            LOGGER.exception(
+                'Unknown error reading %s: %s: %s',
+                self.cache_path, error.__class__.__name__, error)
 
     def save_phrases_cache(self) -> None:
         '''
@@ -517,12 +532,15 @@ class TabSqliteDb:
         try:
             self._phrases_cache['serial_number'] = self._snum
             _cache_path = self.cache_path + '.tmp'
-            # The system may be break stores on rebooting, so
+            # The system may be break during rebooting, so
             # dump to temporary file and then replace it.
-            json.dump(self._phrases_cache, open(_cache_path, 'w'))
+            with open(_cache_path, 'w', encoding='utf-8') as f:
+                json.dump(self._phrases_cache, f)
             os.replace(_cache_path, self.cache_path)
-        except:
-            LOGGER.exception('Unexpected error in save_phrases_cache().')
+        except Exception as error: # pylint: disable=broad-except
+            LOGGER.exception(
+                'Unexpected error in save_phrases_cache(): %s: %s',
+                error.__class__.__name__, error)
 
     def is_chinese(self) -> bool:
         '''
@@ -588,27 +606,27 @@ class TabSqliteDb:
     def create_tables(self, database: str) -> None:
         '''Create tables that contain all phrase'''
         if database == 'main':
-            sqlstr = '''
-            CREATE TABLE IF NOT EXISTS %s.goucima
+            sqlstr = f'''
+            CREATE TABLE IF NOT EXISTS {database}.goucima
             (zi TEXT PRIMARY KEY, goucima TEXT);
-            ''' % database
+            '''
             self.db.execute(sqlstr)
-            sqlstr = '''
-            CREATE TABLE IF NOT EXISTS %s.pinyin
+            sqlstr = f'''
+            CREATE TABLE IF NOT EXISTS {database}.pinyin
             (pinyin TEXT, zi TEXT, freq INTEGER);
-            ''' % database
+            '''
             self.db.execute(sqlstr)
-            sqlstr = '''
-            CREATE TABLE IF NOT EXISTS %s.suggestion
+            sqlstr = f'''
+            CREATE TABLE IF NOT EXISTS {database}.suggestion
             (phrase TEXT, freq INTEGER);
-            ''' %database
+            '''
             self.db.execute(sqlstr)
 
-        sqlstr = '''
-        CREATE TABLE IF NOT EXISTS %s.phrases
+        sqlstr = f'''
+        CREATE TABLE IF NOT EXISTS {database}.phrases
         (id INTEGER PRIMARY KEY, tabkeys TEXT, phrase TEXT,
         freq INTEGER, user_freq INTEGER);
-        ''' % database
+        '''
         self.db.execute(sqlstr)
         self.db.commit()
 
@@ -635,7 +653,7 @@ class TabSqliteDb:
             default_properties=self._default_ime_attributes)
         # The self variables used by tabcreatedb.py need to be updated now:
         self._mlen = int(self.ime_properties.get('max_key_length'))
-        self._is_chinese = self.is_chinese()
+        self.is_db_chinese = self.is_chinese()
         self.user_can_define_phrase = (self.ime_properties.get(
             'user_can_define_phrase').lower() == 'true')
         self.rules = self.get_rules()
@@ -674,7 +692,7 @@ class TabSqliteDb:
                         rules['above'] = int(res.group(2))
                     _cms = res.group(3).split('+')
                     if len(_cms) > self._mlen:
-                        print('rule: "%s" over max key length' %rule)
+                        print(f'rule: "{rule}" over max key length')
                         break
                     for _cm in _cms:
                         cm_res = patt_p.match(_cm)
@@ -683,9 +701,11 @@ class TabSqliteDb:
                                         int(cm_res.group(2))))
                     rules[int(res.group(2))] = cms
                 else:
-                    print('not a legal rule: "%s"' %rule)
-        except Exception:
-            LOGGER.exception('Unexpected error in get_rules().')
+                    print(f'not a legal rule: "{rule}"')
+        except Exception as error: # pylint: disable=broad-except
+            LOGGER.exception(
+                'Unexpected error in get_rules(): %s: %s',
+                error.__class__.__name__, error)
         return rules
 
     def get_possible_tabkeys_lengths(self) -> List[int]:
@@ -755,11 +775,11 @@ class TabSqliteDb:
         '''
         if DEBUG_LEVEL > 1:
             LOGGER.debug('len(phrases)=%s', len(list(phrases)))
-        insert_sqlstr = '''
+        insert_sqlstr = f'''
         INSERT INTO {database}.phrases
         (tabkeys, phrase, freq, user_freq)
         VALUES (:tabkeys, :phrase, :freq, :user_freq);
-        '''.format(database=database)
+        '''
         insert_sqlargs = []
         for (tabkeys, phrase, freq, user_freq) in phrases:
             insert_sqlargs.append({
@@ -790,10 +810,10 @@ class TabSqliteDb:
                 tabkeys, phrase, freq, user_freq)
         if not tabkeys or not phrase:
             return
-        select_sqlstr = '''
+        select_sqlstr = f'''
         SELECT * FROM {database}.phrases
         WHERE tabkeys = :tabkeys AND phrase = :phrase;
-        '''.format(database=database)
+        '''
         select_sqlargs = {'tabkeys': tabkeys, 'phrase': phrase}
         results = self.db.execute(select_sqlstr, select_sqlargs).fetchall()
         if results:
@@ -806,11 +826,11 @@ class TabSqliteDb:
                     select_sqlstr, select_sqlargs, results)
             return
 
-        insert_sqlstr = '''
+        insert_sqlstr = f'''
         INSERT INTO {database}.phrases
         (tabkeys, phrase, freq, user_freq)
         VALUES (:tabkeys, :phrase, :freq, :user_freq);
-        '''.format(database=database)
+        '''
         insert_sqlargs = {
             'tabkeys': tabkeys,
             'phrase': phrase,
@@ -825,8 +845,10 @@ class TabSqliteDb:
             if commit:
                 self.db.commit()
             self.invalidate_phrases_cache(tabkeys)
-        except:
-            LOGGER.exception('Unexpected error in add_phrase()')
+        except Exception as error: # pylint: disable=broad-except
+            LOGGER.exception(
+                'Unexpected error in add_phrase(): %s: %s',
+                error.__class__.__name__, error)
 
     def add_goucima(self, goucimas: Iterable[Tuple[str, str]]) -> None:
         '''Add goucima into database, goucimas is iterable object
@@ -843,8 +865,10 @@ class TabSqliteDb:
             self.db.executemany(sqlstr, sqlargs)
             self.db.commit()
             self.db.execute('PRAGMA wal_checkpoint;')
-        except:
-            LOGGER.exception('Unexpected error in add_goucima().')
+        except sqlite3.Error as error:
+            LOGGER.exception(
+                'Unexpected error in add_goucima(): %s: %s',
+                error.__class__.__name__, error)
 
     def add_pinyin(
             self,
@@ -853,9 +877,10 @@ class TabSqliteDb:
         '''Add pinyin to database, pinyins is a iterable object
         Like: [(zi,pinyin, freq), (zi, pinyin, freq), ...]
         '''
-        sqlstr = '''
-        INSERT INTO %s.pinyin (pinyin, zi, freq) VALUES (:pinyin, :zi, :freq);
-        ''' % database
+        sqlstr = f'''
+        INSERT INTO {database}.pinyin (pinyin, zi, freq)
+        VALUES (:pinyin, :zi, :freq);
+        '''
         count = 0
         for pinyin, zi, freq in pinyins:
             count += 1
@@ -868,11 +893,13 @@ class TabSqliteDb:
             try:
                 self.db.execute(
                     sqlstr, {'pinyin': pinyin, 'zi': zi, 'freq': freq})
-            except Exception:
+            except sqlite3.Error as error:
                 LOGGER.exception(
                     'Error when inserting into pinyin table. '
-                    'count=%s pinyin=%s zi=%s freq=%s',
-                    count, pinyin, zi, freq)
+                    'count=%s pinyin=%s zi=%s freq=%s: '
+                    '%s: %s',
+                    count, pinyin, zi, freq,
+                    error.__class__.__name__, error)
         self.db.commit()
 
     def add_suggestion(
@@ -882,20 +909,21 @@ class TabSqliteDb:
         '''Add suggestion phrase to database, suggestions is a iterable object
         Like: [(phrase, freq), (phrase, freq), ...]
         '''
-        sqlstr = '''
-        INSERT INTO %s.suggestion (phrase, freq) VALUES (:phrase, :freq);
-        ''' % database
+        sqlstr = f'''
+        INSERT INTO {database}.suggestion (phrase, freq) VALUES (:phrase, :freq);
+        '''
         count = 0
         for phrase, freq in suggestions:
             count += 1
             try:
                 self.db.execute(
                     sqlstr, {'phrase': phrase, 'freq': freq})
-            except Exception:
+            except sqlite3.Error as error:
                 LOGGER.exception(
                     'Error when inserting into suggestion table. '
-                    'count=%s phrase=%s freq=%s',
-                    count, phrase, freq)
+                    'count=%s phrase=%s freq=%s: %s: %s',
+                    count, phrase, freq,
+                    error.__class__.__name__, error)
         self.db.commit()
 
     def optimize_database(self) -> None:
@@ -926,7 +954,8 @@ class TabSqliteDb:
         self.db.executescript("VACUUM;")
         self.db.commit()
 
-    def drop_indexes(self, _database: str) -> None:
+    def drop_indexes( # pylint: disable=no-self-use
+            self, _database: str) -> None:
         '''Drop the indexes in the database to reduce its size
 
         We do not use any indexes at the moment, therefore this
@@ -935,7 +964,8 @@ class TabSqliteDb:
         if DEBUG_LEVEL > 1:
             LOGGER.debug('drop_indexes()')
 
-    def create_indexes(self, _database: str, _commit: bool = True) -> None:
+    def create_indexes( # pylint: disable=no-self-use
+            self, _database: str, _commit: bool = True) -> None:
         '''Create indexes for the database.
 
         We do not use any indexes at the moment, therefore
@@ -950,12 +980,14 @@ class TabSqliteDb:
         if DEBUG_LEVEL > 1:
             LOGGER.debug('create_indexes()')
 
-    def big5_code(self, phrase: str) -> bytes:
+    @staticmethod
+    def _big5_code(phrase: str) -> bytes:
         '''
         Encode a string in Big5 or, if that is not possible,
         return something higher than any Big5 code.
 
         :param phrase: String to be encoded in Big5 encoding
+        :returns: Big5-encoded bytes or b'\xff\xff' if unencodable
         '''
         try:
             big5 = phrase.encode('Big5')
@@ -982,18 +1014,19 @@ class TabSqliteDb:
         if engine_name in [
                 'cangjie3', 'cangjie5', 'cangjie-big',
                 'quick-classic', 'quick3', 'quick5']:
-            code_point_function = self.big5_code
-        if self._is_chinese:
-            pinyin_exact_match_function: Callable[[str], int] = lambda x: (
-                - int(typed_tabkeys == x[:-1] and  x[-1] in '!@#$%')
-            )
+            code_point_function = TabSqliteDb._big5_code
+        if self.is_db_chinese:
+            def pinyin_exact_match_function(x: str) -> int:
+                return -int(typed_tabkeys == x[:-1] and x[-1] in '!@#$%')
         else:
-            pinyin_exact_match_function = lambda x: (1)
-        if chinese_mode in (2, 3) and self._is_chinese:
+            def pinyin_exact_match_function( # pylint: disable=unused-argument
+                    x: str) -> int:
+                return 1
+        if chinese_mode in (2, 3) and self.is_db_chinese:
             if chinese_mode == 2:
-                bitmask = (1 << 0) # used in simplified Chinese
+                bitmask = 1 << 0 # used in simplified Chinese
             else:
-                bitmask = (1 << 1) # used in traditional Chinese
+                bitmask = 1 << 1 # used in traditional Chinese
             return sorted(candidates,
                           key=lambda x: (
                               - int(
@@ -1052,7 +1085,7 @@ class TabSqliteDb:
             one_char_condition = ' AND length(phrase)=1 '
 
         if self.user_can_define_phrase or dynamic_adjust:
-            sqlstr = '''
+            sqlstr = f'''
             SELECT tabkeys, phrase, freq, user_freq FROM
             (
                 SELECT tabkeys, phrase, freq, user_freq FROM main.phrases
@@ -1061,12 +1094,12 @@ class TabSqliteDb:
                 SELECT tabkeys, phrase, freq, user_freq FROM user_db.phrases
                 WHERE tabkeys LIKE :tabkeys ESCAPE :escapechar {one_char_condition}
             )
-            '''.format(one_char_condition=one_char_condition)
+            '''
         else:
-            sqlstr = '''
+            sqlstr = f'''
             SELECT tabkeys, phrase, freq, user_freq FROM main.phrases
             WHERE tabkeys LIKE :tabkeys ESCAPE :escapechar {one_char_condition}
-            '''.format(one_char_condition=one_char_condition)
+            '''
         escapechar = '☺'
         for char in '!@#':
             if char not in [single_wildcard_char, multi_wildcard_char]:
@@ -1083,18 +1116,18 @@ class TabSqliteDb:
                 single_wildcard_char, '_')
         if multi_wildcard_char:
             tabkeys_for_like = tabkeys_for_like.replace(
-                multi_wildcard_char, '%%')
+                multi_wildcard_char, '%')
         if auto_wildcard:
-            tabkeys_for_like += '%%'
+            tabkeys_for_like += '%'
         sqlargs = {'tabkeys': tabkeys_for_like, 'escapechar': escapechar}
         if DEBUG_LEVEL > 1:
             LOGGER.debug('sqlstr=%s sqlargs=%s', sqlstr, repr(sqlargs))
         unfiltered_results = self.db.execute(sqlstr, sqlargs).fetchall()
         bitmask = None
         if chinese_mode == 0:
-            bitmask = (1 << 0) # simplified only
+            bitmask = 1 << 0 # simplified only
         elif chinese_mode == 1:
-            bitmask = (1 << 1) # traditional only
+            bitmask = 1 << 1 # traditional only
         if not bitmask:
             results = unfiltered_results
         else:
@@ -1161,9 +1194,9 @@ class TabSqliteDb:
         # which was returned before I simplified the pinyin database table.
         bitmask = None
         if chinese_mode == 0:
-            bitmask = (1 << 0) # simplified only
+            bitmask = 1 << 0 # simplified only
         elif chinese_mode == 1:
-            bitmask = (1 << 1) # traditional only
+            bitmask = 1 << 1 # traditional only
         phrase_frequencies: List[Tuple[str, str, int, int]] = []
         for (pinyin, zi, freq) in results:
             if not bitmask:
@@ -1209,7 +1242,7 @@ class TabSqliteDb:
         if engine_name in [
                 'cangjie3', 'cangjie5', 'cangjie-big',
                 'quick-classic', 'quick3', 'quick5']:
-            code_point_function = self.big5_code
+            code_point_function = TabSqliteDb._big5_code
 
         return sorted(candidates,
                       key=lambda x: (
@@ -1232,19 +1265,23 @@ class TabSqliteDb:
         try:
             sqlstring = (
                 'CREATE TABLE IF NOT EXISTS user_db.desc '
-                + '(name PRIMARY KEY, value);')
+                '(name PRIMARY KEY, value);')
             self.db.executescript(sqlstring)
             sqlstring = 'INSERT OR IGNORE INTO user_db.desc  VALUES (?, ?);'
-            self.db.execute(sqlstring, ('version', DATABASE_VERSION))
+            self.db.execute(
+                sqlstring, ('version', DATABASE_VERSION))
             sqlstring = (
-                'INSERT OR IGNORE INTO user_db.desc  '
-                + 'VALUES (?, DATETIME("now", "localtime"));')
-            self.db.execute(sqlstring, ("create-time", ))
+                "INSERT OR IGNORE INTO user_db.desc  "
+                "VALUES ('create-time', DATETIME('now', 'localtime'));")
+            self.db.execute(sqlstring)
             self.db.commit()
-        except:
-            LOGGER.exception('Unexpected error in generate_userdb_desc().')
+        except Exception as error: # pylint: disable=broad-except
+            LOGGER.exception(
+                'Unexpected error adding description to user_db: %s: %s',
+                 error.__class__.__name__, error)
 
-    def init_user_db(self, db_file: str) -> None:
+    @staticmethod
+    def _init_user_db(db_file: str) -> None:
         '''
         Initialize the user database unless it is an in-memory database
 
@@ -1253,7 +1290,7 @@ class TabSqliteDb:
         '''
         if db_file == ':memory:':
             return
-        if not path.exists(db_file):
+        if not os.path.exists(db_file):
             db = sqlite3.connect(db_file)
             # 20000 pages should be enough to cache the whole database
             db.executescript('''
@@ -1265,10 +1302,12 @@ class TabSqliteDb:
                 PRAGMA journal_mode = WAL;
                 PRAGMA journal_size_limit = 1000000;
                 PRAGMA synchronous = NORMAL;
+                PRAGMA busy_timeout = 5000;
             ''')
             db.commit()
 
-    def get_database_desc(self, db_file: str) -> Optional[Dict[str, str]]:
+    @classmethod
+    def get_database_desc(cls, db_file: str) -> Optional[Dict[str, str]]:
         '''
         Get the description table from the database
 
@@ -1276,7 +1315,7 @@ class TabSqliteDb:
         :type db_file: String
         :rtype: Dictionary
         '''
-        if not path.exists(db_file):
+        if not os.path.exists(db_file):
             return None
         try:
             db = sqlite3.connect(db_file)
@@ -1285,10 +1324,15 @@ class TabSqliteDb:
                 desc[row[0]] = row[1]
             db.close()
             return desc
-        except:
+        except Exception as error: # pylint: disable=broad-except
+            LOGGER.exception(
+                'Unexpected error getting database description: %s: %s',
+                 error.__class__.__name__, error)
             return None
 
-    def get_number_of_columns_of_phrase_table(self, db_file: str) -> int:
+    @classmethod
+    def get_number_of_columns_of_phrase_table(
+            cls, db_file: str) -> Optional[int]:
         '''
         Get the number of columns in the 'phrases' table in
         the database in db_file.
@@ -1307,8 +1351,8 @@ class TabSqliteDb:
         :param db_file: Full path of the database file.
         :rtype: Integer
         '''
-        if not path.exists(db_file):
-            return 0
+        if not os.path.exists(db_file):
+            return None
         try:
             db = sqlite3.connect(db_file)
             tp_res = db.execute(
@@ -1322,7 +1366,11 @@ class TabSqliteDb:
                 tp = res.group(1).split(',')
                 return len(tp)
             return 0
-        except:
+        except Exception as error: # pylint: disable=broad-except
+            LOGGER.exception(
+                'Unexpected error getting number of columns '
+                'of database: %s: %s',
+                 error.__class__.__name__, error)
             return 0
 
     def get_goucima(self, zi: str) -> str:
@@ -1484,10 +1532,10 @@ class TabSqliteDb:
             LOGGER.debug('tabkey=%s phrase=%s', tabkeys, phrase)
         if not tabkeys or not phrase:
             return
-        if self._is_chinese and phrase in CHINESE_NOCHECK_CHARS:
+        if self.is_db_chinese and phrase in CHINESE_NOCHECK_CHARS:
             return
         if not dynamic_adjust:
-            if not self.user_can_define_phrase or not self._is_chinese:
+            if not self.user_can_define_phrase or not self.is_db_chinese:
                 return
             tabkeys = self.parse_phrase(phrase)
             if not tabkeys:
@@ -1515,7 +1563,7 @@ class TabSqliteDb:
                         tabkeys=tabkeys, phrase=phrase, freq=0, user_freq=1,
                         database='user_db')
             else:
-                if not self.user_can_define_phrase or not self._is_chinese:
+                if not self.user_can_define_phrase or not self.is_db_chinese:
                     return
                 tabkeys = self.parse_phrase(phrase)
                 if not tabkeys:
@@ -1565,15 +1613,15 @@ class TabSqliteDb:
         if not phrase:
             return
         if tabkeys:
-            delete_sqlstr = '''
+            delete_sqlstr = f'''
             DELETE FROM {database}.phrases
             WHERE tabkeys = :tabkeys AND phrase = :phrase;
-            '''.format(database=database)
+            '''
         else:
-            delete_sqlstr = '''
+            delete_sqlstr = f'''
             DELETE FROM {database}.phrases
             WHERE phrase = :phrase;
-            '''.format(database=database)
+            '''
         delete_sqlargs = {'tabkeys': tabkeys, 'phrase': phrase}
         self.db.execute(delete_sqlstr, delete_sqlargs)
         if commit:
@@ -1591,9 +1639,10 @@ class TabSqliteDb:
             self.db.commit()
             self.db.execute('PRAGMA wal_checkpoint;')
             self.reset_phrases_cache()
-        except Exception:
+        except Exception as error: # pylint: disable=broad-except
             LOGGER.exception(
-                'Unexpected error removing all phrases from database.')
+                'Unexpected error removing all phrases from database: %s: %s',
+                 error.__class__.__name__, error)
 
     def extract_user_phrases(
             self,
@@ -1659,6 +1708,8 @@ class TabSqliteDb:
                 'Recovered phrases from the very old database: '
                 'phrases=%s', repr(phrases))
             return phrases[:]
-        except:
-            LOGGER.exception('Unexpected error in extract_user_phrases()')
+        except Exception as error: # pylint: disable=broad-except:
+            LOGGER.exception(
+                'Unexpected error extracting user phrases: %s: %s',
+                 error.__class__.__name__, error)
             return []
