@@ -72,7 +72,7 @@ ENGINE_NAME = 'wubi-jidian86'
 
 from gtkcases import TestCases # pylint: disable=import-error
 
-# Need to flush the output against Gtk.main()
+# Need to flush the output against GLib.MainLoop()
 def printflush(sentence: str) -> None:
     try:
         print(sentence, flush=True)
@@ -95,6 +95,7 @@ def printerr(sentence: str) -> None:
     'XDG_SESSION_TYPE is neither "x11" nor "wayland".')
 @unittest.skipIf(Gdk.Display.open('') is None, 'Display cannot be opened.')
 class SimpleGtkTestCase(unittest.TestCase):
+    glib_main_loop: Optional[GLib.MainLoop] = None
     ENGINE_PATH = '/com/redhat/IBus/engines/table/Test/Engine'
     _flag: bool = False
     _gsettings: Optional[Gio.Settings] = None
@@ -125,7 +126,8 @@ class SimpleGtkTestCase(unittest.TestCase):
     def signal_handler(cls, user_data: Any) -> None:
         (signum, original_handler) = user_data
         cls.tearDownClass()
-        Gtk.main_quit()
+        if cls.glib_main_loop is not None:
+            cls.glib_main_loop.quit()
         signal.signal(signum, original_handler)
         cls._flag = True
         assert False, 'signal received: ' + str(signum)
@@ -141,6 +143,12 @@ class SimpleGtkTestCase(unittest.TestCase):
         self.__reset_coming = False
         if self._gsettings is not None:
             self._gsettings.set_int('chinesemode', 4)
+        self.__class__.glib_main_loop = GLib.MainLoop()
+        Gtk.init()
+
+    def tearDown(self) -> None:
+        if self.__class__.glib_main_loop is not None:
+            self.__class__.glib_main_loop.quit()
 
     def register_ibus_engine(self) -> bool:
         self.__bus = IBus.Bus()
@@ -210,7 +218,7 @@ class SimpleGtkTestCase(unittest.TestCase):
             or not IMPORT_TABSQLITEDB_SUCCESSFUL):
             with self.subTest(i='create-engine'):
                 self.fail('NG: ibus-table not installed?')
-            Gtk.main_quit()
+            self.__class__.glib_main_loop.quit()
             return None
         self.__id += 1
         object_path = f'{self.ENGINE_PATH}/{self.__id:d}'
@@ -238,8 +246,8 @@ class SimpleGtkTestCase(unittest.TestCase):
 
     def __engine_focus_in(self, _engine: IBus.Engine) -> None:
         if self.__test_index == len(TestCases['tests']):
-            if DONE_EXIT:
-                Gtk.main_quit()
+            if DONE_EXIT and self.__class__.glib_main_loop is not None:
+                self.__class__.glib_main_loop.quit()
             return
         # Workaround because focus-out resets the preedit text
         # ibus_bus_set_global_engine() calls bus_input_context_set_engine()
@@ -261,8 +269,8 @@ class SimpleGtkTestCase(unittest.TestCase):
     def __entry_focus_in_event_cb(
             self, entry: Gtk.Entry, event: Gdk.EventFocus) -> bool:
         if self.__test_index == len(TestCases['tests']):
-            if DONE_EXIT:
-                Gtk.main_quit()
+            if DONE_EXIT and self.__class__.glib_main_loop is not None:
+                self.__class__.glib_main_loop.quit()
             return False
         self.__bus.set_global_engine_async(ENGINE_NAME,
                                            -1, None, self.__set_engine_cb)
@@ -290,8 +298,8 @@ class SimpleGtkTestCase(unittest.TestCase):
         if len(preedit_str) == 0:
             return
         if self.__test_index == len(TestCases['tests']):
-            if DONE_EXIT:
-                Gtk.main_quit()
+            if DONE_EXIT and self.__class__.glib_main_loop is not None:
+                self.__class__.glib_main_loop.quit()
             return
         self.__preedit_index += 1
         if self.__preedit_index != self.__get_test_condition_length('preedit'):
@@ -384,16 +392,16 @@ class SimpleGtkTestCase(unittest.TestCase):
         if cases['string'] == self.__inserted_text:
             printflush(f'OK: {self.__test_index} "{self.__inserted_text}"')
         else:
-            if DONE_EXIT:
-                Gtk.main_quit()
+            if DONE_EXIT and self.__class__.glib_main_loop is not None:
+                self.__class__.glib_main_loop.quit()
             with self.subTest(i=self.__test_index):
                 self.fail(f'NG: {self.__test_index:d} '
                            f'"{str(cases["string"])}" "{self.__inserted_text}"')
         self.__inserted_text = ''
         self.__test_index += 1
         if self.__test_index == len(TestCases['tests']):
-            if DONE_EXIT:
-                Gtk.main_quit()
+            if DONE_EXIT and self.__class__.glib_main_loop is not None:
+                self.__class__.glib_main_loop.quit()
             return
         self.__commit_done = True # pylint: disable=unused-private-member
         self.__entry.set_text('')
@@ -403,7 +411,8 @@ class SimpleGtkTestCase(unittest.TestCase):
     def create_window(self) -> None:
         window = Gtk.Window(type=Gtk.WindowType.TOPLEVEL)
         self.__entry = entry = Gtk.Entry()
-        window.connect('destroy', Gtk.main_quit)
+        if self.__class__.glib_main_loop is not None:
+            window.connect('destroy', self.__class__.glib_main_loop.quit)
         entry.connect('focus-in-event', self.__entry_focus_in_event_cb)
         entry.connect('preedit-changed', self.__entry_preedit_changed_cb)
         buffer = entry.get_buffer()
@@ -414,7 +423,8 @@ class SimpleGtkTestCase(unittest.TestCase):
     def main(self) -> None: # pylint: disable=no-self-use
         # Some ATK relative warnings are called during launching GtkWindow.
         flags = GLib.log_set_always_fatal(GLib.LogLevelFlags.LEVEL_CRITICAL)
-        Gtk.main()
+        if self.__class__.glib_main_loop is not None:
+            self.__class__.glib_main_loop.run()
         GLib.log_set_always_fatal(flags)
 
     def test_typing(self) -> None:
